import os
import logging
from datetime import timedelta

from flask import Flask
from flask_sqlalchemy import SQLAlchemy
from flask_login import LoginManager
from flask_migrate import Migrate
from sqlalchemy.orm import DeclarativeBase
from werkzeug.middleware.proxy_fix import ProxyFix

# Configure logging
logging.basicConfig(level=logging.INFO)

# Declarative Base for SQLAlchemy >= 3.x
class Base(DeclarativeBase):
    pass

# Initialize SQLAlchemy instance
db = SQLAlchemy(model_class=Base)

# Initialize Flask app
app = Flask(__name__)
app.secret_key = os.environ.get("SESSION_SECRET", "your-secret-key-here")
app.wsgi_app = ProxyFix(app.wsgi_app, x_proto=1, x_host=1)

# Max file upload size (16 MB)
app.config['MAX_CONTENT_LENGTH'] = 16 * 1024 * 1024

# Read DB URL from environment or use SQLite as fallback
database_url = os.environ.get("DATABASE_URL")
if database_url:
    if database_url.startswith("mysql://"):
        # Flask-SQLAlchemy prefers pymysql
        database_url = database_url.replace("mysql://", "mysql+pymysql://mastoore_mer:'Mer@09121'@localhost/mastoore_mer", 1)
    
    logging.info(f"Using database from environment: {database_url.split('@')[0]}@****")

    if "mysql" in database_url:
        app.config["SQLALCHEMY_ENGINE_OPTIONS"] = {
            "pool_recycle": 3600,
            "pool_pre_ping": True,
            "pool_size": 3,
            "max_overflow": 2,
            "echo": False,
            "connect_args": {
                "charset": "utf8mb4",
                "init_command": "SET sql_mode='STRICT_TRANS_TABLES'",
            }
        }
    else:
        # PostgreSQL or other
        app.config["SQLALCHEMY_ENGINE_OPTIONS"] = {
            "pool_recycle": 300,
            "pool_pre_ping": True,
            "pool_size": 5,
            "max_overflow": 10,
            "echo": False,
        }
else:
    # SQLite fallback for local development
    database_url = "sqlite:///instance/shipping.db"
    logging.warning("DATABASE_URL not set. Using SQLite for development.")

    app.config["SQLALCHEMY_ENGINE_OPTIONS"] = {
        "pool_pre_ping": True,
        "echo": False,
    }

# Apply database config
app.config["SQLALCHEMY_DATABASE_URI"] = database_url
app.config["SQLALCHEMY_TRACK_MODIFICATIONS"] = False

# Initialize extensions
db.init_app(app)
migrate = Migrate(app, db)

# Initialize Flask-Login
login_manager = LoginManager()
login_manager.init_app(app)
login_manager.login_view = 'login'
login_manager.login_message = 'يرجى تسجيل الدخول للوصول إلى هذه الصفحة.'
login_manager.login_message_category = 'info'

@login_manager.user_loader
def load_user(user_id):
    from models import Admin
    return Admin.query.get(int(user_id))

# Create database and default admin if necessary
with app.app_context():
    try:
        import models  # Must import to register tables

        db.create_all()
        logging.info('Database tables created or verified.')

        from models import Admin
        admin = Admin.query.filter_by(username='admin').first()
        if not admin:
            admin = Admin()
            admin.username = 'admin'
            admin.is_super_admin = True
            admin.set_password('admin123')  # Replace with secure password
            admin.set_permissions({
                'home': True,
                'shipments': True,
                'tracking': True,
                'reports': True,
                'expenses': True,
                'add_shipment': True,
                'settings': True
            })
            db.session.add(admin)
            db.session.commit()
            logging.info('Default super admin created: admin/admin123')
        elif not admin.is_super_admin:
            admin.is_super_admin = True
            db.session.commit()
            logging.info('Existing admin updated to super admin.')

    except Exception as e:
        logging.error(f'Database initialization error: {str(e)}')

# Import your routes here
import routes  # noqa: F401

# REQUIRED by Passenger to launch the app
application = app
